import React,{ PureComponent }  from 'react';
import {
    StyleSheet,
    View,
    Text,
    TouchableWithoutFeedback,
    Dimensions,
    Image,
    Animated,
    ListView,
    Platform
} from 'react-native';

import Refreshing from './Refreshing';
import LoadingMore from './LoadingMore';
import PullToRefreshListView from '../iospulltorefreshlistview/PullToRefreshListView';

/**
 * 下拉头部最大高度
 * @type {number}
 */
const MAX_REFRESH_HEIGHT = 80;

export default class RefreshListView extends PureComponent {

    static defaultProps = {
        enabledPullDown: true,
        enabledPullUp: false
    }

    /**
     *
     * @type {null}
     * @private
     */
    _ds = null;
    /**
     *
     * @type {Array}
     * @private
     */
    _dataList = [];

    /**
     * 是否正在下拉刷新
     * @type {boolean}
     */
    isRefreshing = false;
    /**
     * 是否正在加载更多
     * @type {boolean}
     */
    isLoadMore = false;

    constructor(props) {
        super(props);

        const { dataSource, dataList } = this.props;

        if(dataSource){
            this.state = {
                dataSource:dataSource,
            }
        }else{
            this._dataList.addAll(dataList);
            this._ds = new ListView.DataSource({rowHasChanged: (r1, r2) => r1 !== r2});
            this.state={
                dataSource:this._ds.cloneWithRows(this._dataList),
            }
        }
    }

    /**
     * 停止滚动
     */
    stopScroll = ()=>{
        this._pullToRefreshListView && this._pullToRefreshListView.stopScroll();
    }

    /**
     *
     * @param dataSource
     */
    setDataSource = (dataSource : ListView.DataSource)=>{
        this.setState({ dataSource });
    }

    /**
     *
     * @param item
     * @param index
     */
    addItemData = (item : any, index : number)=>{
        if(this._ds){
            this._dataList.add(item, index);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     *
     * @param item
     */
    addAllItemData(itemArray : Array){
        if(this._ds){
            this._dataList.addAll(itemArray);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     *
     * @param position
     */
    removeItemDataAtPosition = (position : number)=>{
        if(this._ds){
            if(typeof position === 'string'){
                position = Number(position);
            }
            this._dataList.remove(position);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     *
     * @param position
     */
    removeItemData = (item : any)=>{
        if(this._ds){
            this._dataList.remove(item);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     * 清空数据
     */
    clear = ()=>{
        if(this._ds){
            this._dataList = [];
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     * 清空数据
     */
    clearAndAddAll = (itemArray : Array = [])=>{
        if(this._ds){
            this._dataList = [];
            this._dataList.addAll(itemArray);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     * 数据总数
     * @returns {number}
     */
    getItemCount = () : number =>{
        return this.state.dataSource.getRowCount();
    }

    /**
     * 开始下拉刷新
     */
    beginRefreshing = () => {
        if (!this.isRefreshing && !this.isLoadMore) {
            this._pullToRefreshListView && this._pullToRefreshListView.beginRefresh();
            this.isRefreshing = true;
        }
    }

    /**
     * 结束下拉刷新
     */
    endRefreshing = () => {
        if (this.isRefreshing) {
            this._pullToRefreshListView && this._pullToRefreshListView.endRefresh();
            this.isRefreshing = false;
        }
    }

    /**
     *
     * @param hasLoadAll    是否已经加载所有数据  true：底部显示没有更多了  false: 加载中...
     */
    endLoadMore = (hasLoadAll : boolean = false, ingoreLoadMore : boolean = false) => {
        if (this.isLoadMore || ingoreLoadMore) {
            this._pullToRefreshListView && this._pullToRefreshListView.endLoadMore(hasLoadAll);
            this.isLoadMore = false;
        }
    }

    /**
     * 头部刷新 View
     * @param txt
     * @returns {XML}
     * @private
     */
    _renderHeaderRefreshView = (txt : String, play = false)=>{
        return (
            <Refreshing
                key={'headerRefreshView'}
                style={{height: 50, marginBottom: (MAX_REFRESH_HEIGHT - 50) / 2}}
                play={play}
                isShow={true}
                text={txt}
            />
        );
    }

    /**
     *
     * @param viewState
     * @returns {*}
     * @private
     */
    _renderHeader = (viewState) => {
        if (!this.props.enabledPullDown) {
            return null;
        }
        let {pullState, pullDistancePercent} = viewState;

        let {refresh_none, refresh_idle, will_refresh, refreshing,} = PullToRefreshListView.constants.viewState;
        pullDistancePercent = Math.round(pullDistancePercent * 100);
        this.props.viewStateChange && this.props.viewStateChange(pullState);
        switch(pullState) {
            case refresh_none:
                this.isRefreshing = false;
                return this._renderHeaderRefreshView('下拉刷新');
            case refresh_idle:
                this.isRefreshing = false;
                return this._renderHeaderRefreshView('下拉刷新');
            case will_refresh:
                this.isRefreshing = false;
                return this._renderHeaderRefreshView('松开刷新');
            case refreshing:
                this.isRefreshing = true;
                return this._renderHeaderRefreshView('加载中', true);
        }
    }

    /**
     * 加载更多 View
     * @param txt
     * @param hasLoadAll    是否已经加载全部数据   true：显示没有更多了 false：显示正在加载中...
     * @returns {XML}
     * @private
     */
    _renderLoadMoreView = (txt : String = '盟管家正在加载', hasLoadAll : Boolean = false)=>{

        const { loadAllLineStyle } = this.props;

        return (
            <LoadingMore
                key={'loadMoreView'}
                style={{height: 50}}
                isShow={true}
                loadAllLineStyle={loadAllLineStyle}
                hasLoadAll={hasLoadAll}
                txt={txt}
            />
        );
    }

    _renderFooter = (viewState) => {
        if (!this.props.enabledPullUp) {
            return null;
        }
        let {pullState, pullDistancePercent} = viewState;
        let {load_more_none, load_more_idle, will_load_more, loading_more, loaded_all, } = PullToRefreshListView.constants.viewState
        this.props.viewStateChange && this.props.viewStateChange(pullState);

        switch(pullState) {
            case load_more_none:
                this.isLoadMore = false;
                return this._renderLoadMoreView();
            case load_more_idle:
                this.isLoadMore = false;
                return this._renderLoadMoreView();
            case will_load_more:
                this.isLoadMore = false;
                return this._renderLoadMoreView();
            case loading_more:
                this.isLoadMore = true;
                return this._renderLoadMoreView();
            case loaded_all:
                this.isLoadMore = false;
                return this._renderLoadMoreView(null, true);
        }
    }

    /**
     *
     * @param props
     */
    setNativeProps = (props) => {
        this._pullToRefreshListView && this._pullToRefreshListView.setNativeProps(props)
    }

    render() {

        const { pageSize = 20, initialListSize = 20, renderHeader, enabledPullUp, ...otherProps } = this.props;

        let autoLoadMore = enabledPullUp;

        return (
            <PullToRefreshListView
                ref={ (component) => this._pullToRefreshListView = component }
                viewType={PullToRefreshListView.constants.viewType.listView}
                enableEmptySections={true}
                pullDownDistance={MAX_REFRESH_HEIGHT}
                pullDownStayDistance={MAX_REFRESH_HEIGHT}
                pullUpDistance={MAX_REFRESH_HEIGHT}
                pullUpStayDistance={MAX_REFRESH_HEIGHT}
                {...otherProps}
                renderHeader={this._renderHeader}
                renderFooter={this._renderFooter}
                renderListHeader={renderHeader}
                autoLoadMore={autoLoadMore}
                enabledPullUp={enabledPullUp}
                pageSize={pageSize}
                initialListSize={initialListSize}
                dataSource={this.state.dataSource}
            />
        );
    }
}
