import React, { PureComponent } from 'react';
import ReactNative, {
    View,
	Text,
	Platform,
    requireNativeComponent,
    UIManager,
	ListView,
    SectionList,
} from 'react-native';

import PropTypes from 'prop-types';


import Refreshing from './Refreshing';

/**
 *
 */
const PullRefreshHeaderContainerView = requireNativeComponent('AndroidPullRefreshHeaderContainerView');
/**
 *
 */
const PullRefreshFooterContainerView = requireNativeComponent('AndroidPullRefreshFooterContainerView');
/**
 *
 */
const PullRefreshFooterEndContainerView = requireNativeComponent('AndroidPullRefreshFooterEndContainerView');

/**
 *
 */
const NativePullToRefreshContainer = requireNativeComponent('AndroidPullToRefreshContainer', PullToRefreshContainer);

/**
 *
 */
const CONSTANTS_REFRESH_MODE = UIManager.AndroidPullToRefreshContainer.Constants.REFRESH_MODE;

/**
 *
 */
class PullToRefreshContainer extends PureComponent{

    /**
     *
     * @type {{useRefreshViewDefault: React$PropType$Primitive.<boolean>, refreshMode: *, headerRefreshView, footerRefreshView, onPullEvent, onPullDownToRefresh, onPullUpToRefresh}}
     */
    static propTypes = {
        useRefreshViewDefault:PropTypes.bool, //是否原生使用默认的刷新view  默认：true
        refreshMode:PropTypes.oneOf([CONSTANTS_REFRESH_MODE.DISABLED, CONSTANTS_REFRESH_MODE.PULL_FROM_START, CONSTANTS_REFRESH_MODE.PULL_FROM_END, CONSTANTS_REFRESH_MODE.BOTH]),   // 下拉/上拉刷新模式 默认: DISABLED
        headerRefreshView:PropTypes.element,   //自定义下拉刷新view
        footerRefreshView:PropTypes.element,   //自定义上拉刷新view
        footerEndView:PropTypes.element,   //自定义加载完成view
        onPullEvent:PropTypes.func,   //下拉状态通知
        onPullDownToRefresh:PropTypes.func,   //下拉刷新
        onPullUpToRefresh:PropTypes.func,   //上拉加载
    }

    /**
     *
     * @type {{useRefreshViewDefault: boolean, refreshMode: *}}
     */
    static defaultProps = {
        useRefreshViewDefault:false,
        refreshMode:CONSTANTS_REFRESH_MODE.PULL_FROM_START,
    }

    /**
	 *
     */
    static REFRESH_MODE = CONSTANTS_REFRESH_MODE;

    /**
     * 是否下拉刷新
     * @type {null}
     * @private
     */
    _isPullDownToRefresh = null;

	constructor(props){
		super(props);
	}

    /**
     * 刷新完成
     */
    onRefreshComplete = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidPullToRefreshContainer.Commands.onRefreshComplete, null);
        }
    }

    /**
     * 自动下拉刷新
     */
    setTopRefreshing = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidPullToRefreshContainer.Commands.setTopRefreshing, null);
        }
    }

    /**
     * 底部加载结束
     */
    onLoadMoreComplete = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidPullToRefreshContainer.Commands.onLoadMoreComplete, null);
        }
    }

    /**
     * 没有更多
     */
    onLoadMoreCompleteAndNoData = ()=>{
        let listViewHandle = this._getListViewHandle();
        if(listViewHandle){
            UIManager.dispatchViewManagerCommand(listViewHandle, UIManager.AndroidPullToRefreshContainer.Commands.onLoadMoreCompleteAndNoData, null);
        }
    }

    /**
     *
     * @private
     */
    _getListViewHandle = ()=>{
        if(this.refs.nativePullToRefreshContainer){
            return ReactNative.findNodeHandle(this.refs.nativePullToRefreshContainer);
        }
    }

    /**
     * 下拉/上拉 状态通知
     * @param e
     * @private
     */
    _onPullEvent = (e)=>{
        this._isPullDownToRefresh = false;

        this.props.onPullEvent && this.props.onPullEvent(e.nativeEvent);

        let { currMode, pullState } = e.nativeEvent;

        let refreshTxt = '';

        if(currMode === 'PULL_FROM_START'){ //下拉
            if(pullState === 'PULL_TO_REFRESH'){
                refreshTxt = '下拉即可刷新';
            }else if(pullState === 'RELEASE_TO_REFRESH'){
                refreshTxt = '释放即可刷新';
            }else if(pullState === 'RESET'){
                refreshTxt = '下拉即可刷新';
            }else if(pullState === 'REFRESHING'){
                refreshTxt = '刷新中...';
                this._isPullDownToRefresh = true;
            }
        }else{  //上拉
            if(pullState === 'PULL_TO_REFRESH'){
                refreshTxt = '上拉即可刷新';
            }else if(pullState === 'RELEASE_TO_REFRESH'){
                refreshTxt = '释放即可刷新';
            }else if(pullState === 'RESET'){
                refreshTxt = '上拉即可刷新';
            }else if(pullState === 'REFRESHING'){
                refreshTxt = '刷新中...';
            }
        }

        console.log('refreshTxt============> ', refreshTxt);
    }

	render(){

        const { headerRefreshView, footerRefreshView, footerEndView, refreshMode, useRefreshViewDefault, children, ...otherProps } = this.props;

        let pullRefreshHeaderContainerView;
        //下拉刷新view
        if(!useRefreshViewDefault && (refreshMode === CONSTANTS_REFRESH_MODE.BOTH || refreshMode === CONSTANTS_REFRESH_MODE.PULL_FROM_START) && headerRefreshView){
            pullRefreshHeaderContainerView = (
                <PullRefreshHeaderContainerView style={{backgroundColor:'blue'}}>
                    {headerRefreshView}
                </PullRefreshHeaderContainerView>
            );
        }

        //上拉刷新view
        let pullRefreshFooterContainerView = null;
        if(!useRefreshViewDefault && (refreshMode === CONSTANTS_REFRESH_MODE.BOTH || refreshMode === CONSTANTS_REFRESH_MODE.PULL_FROM_END) && footerRefreshView){
            let pullRefreshFooterEndContainerView = null;
            if(footerEndView){
                pullRefreshFooterEndContainerView = (
                    <PullRefreshFooterEndContainerView style={{backgroundColor:'red'}}>
                        {footerEndView}
                    </PullRefreshFooterEndContainerView>
                );
            }

            pullRefreshFooterContainerView = (
                <PullRefreshFooterContainerView style={{backgroundColor:'yellow'}}>
                    {footerRefreshView}
                    {pullRefreshFooterEndContainerView}
                </PullRefreshFooterContainerView>
            );
        }

		return (
            <NativePullToRefreshContainer
                ref={'nativePullToRefreshContainer'}
                {...otherProps}
                refreshMode={refreshMode}
                useRefreshViewDefault={useRefreshViewDefault}
                onPullEvent={this._onPullEvent}
            >
                { pullRefreshHeaderContainerView }

                { children }

                { pullRefreshFooterContainerView }

            </NativePullToRefreshContainer>
        );
	}

}

/**
 *
 */
export default class RefreshListView extends PureComponent {

    /**
	 *
     * @type {{useRefreshViewDefault: React$PropType$Primitive.<boolean>, enabledPullDown: React$PropType$Primitive.<boolean>, enabledPullUp: React$PropType$Primitive.<boolean>, onRefresh, onLoadMore}}
     */
    static propTypes = {
        enabledPullDown:PropTypes.bool,		//是否开启下拉刷新
        enabledPullUp:PropTypes.bool,		//是否开启上拉加载
        onRefresh:PropTypes.func,   //下拉刷新
        onLoadMore:PropTypes.func,   //上拉加载
    }

    /**
	 *
     * @type {{enabledPullDown: boolean, enabledPullUp: boolean}}
     */
    static defaultProps = {
        enabledPullDown: false,
        enabledPullUp: false
    }

    /**
     * 下拉/上拉 状态通知
     * @param e
     * @private
     */
    _onPullEvent = ({ currMode, pullState })=>{
        let refreshTxt = '';

        if(currMode === 'PULL_FROM_START'){ //下拉
            if(pullState === 'PULL_TO_REFRESH'){
                refreshTxt = '下拉即可刷新';
            }else if(pullState === 'RELEASE_TO_REFRESH'){
                refreshTxt = '释放即可刷新';
            }else if(pullState === 'RESET'){
                refreshTxt = '下拉即可刷新';
            }else if(pullState === 'REFRESHING'){
                refreshTxt = '刷新中...';
            }
        }else{  //上拉
            if(pullState === 'PULL_TO_REFRESH'){
                refreshTxt = '上拉即可刷新';
            }else if(pullState === 'RELEASE_TO_REFRESH'){
                refreshTxt = '释放即可刷新';
            }else if(pullState === 'RESET'){
                refreshTxt = '上拉即可刷新';
            }else if(pullState === 'REFRESHING'){
                refreshTxt = '刷新中...';
            }
        }
        console.log('_onPullEvent refreshTxt============> ', refreshTxt);
    }

    /**
	 * 下拉刷新开始
     * @private
     */
    _onPullDownToRefresh = ()=>{
    	console.log('下拉刷新开始===========>');
        this.props.onRefresh && this.props.onRefresh();
	}

    /**
	 * 上拉刷新开始
     * @private
     */
    _onPullUpToRefresh = ()=>{
        console.log('上拉刷新开始===========>');
        this.props.onLoadMore && this.props.onLoadMore();
	}

    /**
	 * 刷新结束
     */
    endRefreshing = () => {
    	this.refs.pullToRefreshContainer && this.refs.pullToRefreshContainer.onRefreshComplete();
    }

    /**
	 * 自动下拉刷新
     */
    beginRefreshing = () => {
        this.refs.pullToRefreshContainer && this.refs.pullToRefreshContainer.setTopRefreshing();
    }

    /**
     * 上拉加载结束
     * @param isNoMore
     */
    endLoadMore = (isNoMore : boolean = false) => {
        if(this.refs.pullToRefreshContainer && !this.refs.pullToRefreshContainer._isPullDownToRefresh){
            if(isNoMore){	//没有更多
                this.refs.pullToRefreshContainer.onLoadMoreCompleteAndNoData();
            }else{	//加载更多结束
                this.refs.pullToRefreshContainer.onLoadMoreComplete();
            }
        }
    }

	render() {

		const { enabledPullUp, enabledPullDown, onRefresh, onLoadMore, ...otherProps } = this.props;

		let refreshMode = PullToRefreshContainer.REFRESH_MODE.DISABLED;
		if(!!enabledPullDown && !!enabledPullUp){	//支持下拉、上拉
			refreshMode = PullToRefreshContainer.REFRESH_MODE.BOTH;
		}else if(!!enabledPullDown){	//下拉
			refreshMode = PullToRefreshContainer.REFRESH_MODE.PULL_FROM_START;
		}else if(!!enabledPullUp){	//上拉
			refreshMode = PullToRefreshContainer.REFRESH_MODE.PULL_FROM_END;
		}

		let refreshControl = (
			<PullToRefreshContainer
				ref={'pullToRefreshContainer'}
				refreshMode={refreshMode}
				onPullEvent={this._onPullEvent}
				onPullDownToRefresh={this._onPullDownToRefresh}
				onPullUpToRefresh={this._onPullUpToRefresh}
                headerRefreshView={<Refreshing isShow={true} style={{height:50}} text="下拉刷新" />}
                footerRefreshView={<Refreshing isShow={true} style={{height:50}} text="上拉加载" />}
                footerEndView={
                    <View opacity={1} style={{height: 50, justifyContent: 'center', alignItems: 'center', flexDirection:'row'}}>
                        <View style={{backgroundColor:'#d7d7d7', height:0.5, flex: 1}} />
                        <Text style={{fontSize: 14, color:'#999', marginLeft: 10, marginRight: 10}}>没有更多了</Text>
                        <View style={{backgroundColor:'#d7d7d7', height:0.5, flex: 1}} />
                    </View>
                }
			/>
		);

		return (
			<ListView
				{...otherProps}
				refreshControl={refreshControl}
				initialListSize={15}
				pageSize={10}
			/>
		);
	}
}